@@ -23,21 +23,19 @@ import (
 	"time"
 
 	// Inject the fakes for informers this reconciler depends on.
-	_ "github.com/knative/pkg/injection/informers/kubeinformers/corev1/endpoints/fake"
-	_ "github.com/knative/pkg/injection/informers/kubeinformers/corev1/service/fake"
-	_ "github.com/knative/serving/pkg/client/injection/informers/networking/v1alpha1/serverlessservice/fake"
+	_ "knative.dev/pkg/client/injection/kube/informers/core/v1/endpoints/fake"
+	_ "knative.dev/pkg/client/injection/kube/informers/core/v1/service/fake"
+	_ "knative.dev/serving/pkg/client/injection/informers/networking/v1alpha1/serverlessservice/fake"
 
-	"github.com/knative/pkg/configmap"
-	"github.com/knative/pkg/controller"
-	logtesting "github.com/knative/pkg/logging/testing"
-	"github.com/knative/pkg/ptr"
-	"github.com/knative/pkg/system"
-	"github.com/knative/serving/pkg/activator"
-	"github.com/knative/serving/pkg/apis/networking"
-	nv1a1 "github.com/knative/serving/pkg/apis/networking/v1alpha1"
-	rpkg "github.com/knative/serving/pkg/reconciler"
-	"github.com/knative/serving/pkg/reconciler/serverlessservice/resources"
-	presources "github.com/knative/serving/pkg/resources"
+	"knative.dev/pkg/configmap"
+	"knative.dev/pkg/controller"
+	"knative.dev/pkg/ptr"
+	"knative.dev/pkg/system"
+	"knative.dev/serving/pkg/apis/networking"
+	nv1a1 "knative.dev/serving/pkg/apis/networking/v1alpha1"
+	rpkg "knative.dev/serving/pkg/reconciler"
+	"knative.dev/serving/pkg/reconciler/serverlessservice/resources"
+	presources "knative.dev/serving/pkg/resources"
 
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -47,13 +45,12 @@ import (
 	"k8s.io/apimachinery/pkg/util/intstr"
 	clientgotesting "k8s.io/client-go/testing"
 
-	. "github.com/knative/pkg/reconciler/testing"
-	. "github.com/knative/serving/pkg/reconciler/testing/v1alpha1"
-	. "github.com/knative/serving/pkg/testing"
+	. "knative.dev/pkg/reconciler/testing"
+	. "knative.dev/serving/pkg/reconciler/testing/v1alpha1"
+	. "knative.dev/serving/pkg/testing"
 )
 
 func TestNewController(t *testing.T) {
-	defer logtesting.ClearAll()
 	ctx, _ := SetupFakeContext(t)
 	c := NewController(ctx, configmap.NewStaticWatcher())
 	if c == nil {
@@ -78,52 +75,70 @@ func TestReconcile(t *testing.T) {
 		Name: "steady state",
 		Key:  "steady/state",
 		Objects: []runtime.Object{
-			SKS("steady", "state", markHappy, WithPubService, WithPrivateService("state-fsdf"),
-				WithDeployRef("bar")),
+			SKS("steady", "state", markHappy, WithPubService, WithPrivateService, WithDeployRef("bar")),
 			deploy("steady", "bar"),
 			svcpub("steady", "state"),
-			svcpriv("steady", "state", svcWithName("state-fsdf")),
-			endpointspub("steady", "state", WithSubsets),
-			endpointspriv("steady", "state", WithSubsets, epsWithName("state-fsdf")),
+			svcpriv("steady", "state"),
+			endpointspub("steady", "state", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+			endpointspriv("steady", "state", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 	}, {
+		// This is the case for once we are scaled to zero.
 		Name: "steady switch to proxy mode",
 		Key:  "steady/to-proxy",
 		Objects: []runtime.Object{
-			SKS("steady", "to-proxy", markHappy, WithPubService, WithPrivateService("to-proxy-deadbeef"),
-				WithDeployRef("bar"), WithProxyMode),
+			SKS("steady", "to-proxy", markHappy, WithPubService, WithPrivateService,
+				WithDeployRef("bar"), withProxyMode),
 			deploy("steady", "bar"),
 			svcpub("steady", "to-proxy"),
-			svcpriv("steady", "to-proxy", svcWithName("to-proxy-deadbeef")),
-			endpointspub("steady", "to-proxy", withOtherSubsets),
-			endpointspriv("steady", "to-proxy", epsWithName("to-proxy-deadbeed")),
+			svcpriv("steady", "to-proxy"),
+			endpointspub("steady", "to-proxy", withOtherSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+			endpointspriv("steady", "to-proxy"),
 			activatorEndpoints(WithSubsets),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("steady", "to-proxy", WithDeployRef("bar"),
-				markNoEndpoints, WithProxyMode, WithPubService, WithPrivateService("to-proxy-deadbeef")),
+			Object: SKS("steady", "to-proxy", WithDeployRef("bar"), markNoEndpoints,
+				withProxyMode, WithPubService, WithPrivateService),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: endpointspub("steady", "to-proxy", WithSubsets),
+			Object: endpointspub("steady", "to-proxy", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "steady/to-proxy"`),
 		},
+	}, {
+		// This is the case for once we are proxying for unsufficient burst capacity.
+		// It should be a no-op.
+		Name: "steady switch to proxy mode with endpoints",
+		Key:  "steady/to-proxy",
+		Objects: []runtime.Object{
+			SKS("steady", "to-proxy", markHappy, WithPubService, WithPrivateService,
+				WithDeployRef("bar"), withProxyMode),
+			deploy("steady", "bar"),
+			svcpub("steady", "to-proxy"),
+			svcpriv("steady", "to-proxy"),
+			endpointspub("steady", "to-proxy", withOtherSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+			endpointspriv("steady", "to-proxy", WithSubsets),
+			activatorEndpoints(WithSubsets),
+		},
+		WantUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: endpointspub("steady", "to-proxy", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+		}},
 	}, {
 		Name: "many-private-services",
 		Key:  "many/privates",
 		Objects: []runtime.Object{
-			SKS("many", "privates", markHappy, WithPubService, WithPrivateService("privates-elegance-required"),
+			SKS("many", "privates", markHappy, WithPubService, WithPrivateService,
 				WithDeployRef("bar")),
 			deploy("many", "bar"),
 			svcpub("many", "privates"),
-			svcpriv("many", "privates", svcWithName("privates-elegance-required")),
+			svcpriv("many", "privates"),
 			svcpriv("many", "privates", svcWithName("privates-brutality-is-here")),
 			svcpriv("many", "privates", svcWithName("privates-uncharacteristically-pretty"),
 				WithK8sSvcOwnersRemoved), // unowned, should remain.
-			endpointspub("many", "privates", WithSubsets),
-			endpointspriv("many", "privates", WithSubsets, epsWithName("privates-elegance-required")),
+			endpointspub("many", "privates", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+			endpointspriv("many", "privates", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantDeletes: []clientgotesting.DeleteActionImpl{{
@@ -143,12 +158,12 @@ func TestReconcile(t *testing.T) {
 		Key:  "public/svc-change",
 		Objects: []runtime.Object{
 			SKS("public", "svc-change", WithPubService, WithSKSReady,
-				WithPrivateService("svc-change-feedbeef"), WithDeployRef("bar")),
+				WithPrivateService, WithDeployRef("bar")),
 			deploy("public", "bar"),
 			svcpub("public", "svc-change", withTimeSelector),
-			svcpriv("public", "svc-change", svcWithName("svc-change-feedbeef")),
-			endpointspub("public", "svc-change", WithSubsets),
-			endpointspriv("public", "svc-change", WithSubsets, epsWithName("svc-change-feedbeef")),
+			svcpriv("public", "svc-change"),
+			endpointspub("public", "svc-change", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+			endpointspriv("public", "svc-change", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
@@ -159,18 +174,18 @@ func TestReconcile(t *testing.T) {
 		Key:  "private/svc-change",
 		Objects: []runtime.Object{
 			SKS("private", "svc-change", markHappy, WithPubService,
-				WithPrivateService("svc-change-fade"), WithDeployRef("baz")),
+				WithPrivateService, WithDeployRef("baz")),
 			deploy("private", "baz"),
 			svcpub("private", "svc-change"),
-			svcpriv("private", "svc-change", withTimeSelector, svcWithName("svc-change-fade")),
-			endpointspub("private", "svc-change", withOtherSubsets),
-			endpointspriv("private", "svc-change", WithSubsets, epsWithName("svc-change-fade")),
+			svcpriv("private", "svc-change", withTimeSelector),
+			endpointspub("private", "svc-change", withOtherSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+			endpointspriv("private", "svc-change", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: svcpriv("private", "svc-change", svcWithName("svc-change-fade")),
+			Object: svcpriv("private", "svc-change"),
 		}, {
-			Object: endpointspub("private", "svc-change", WithSubsets),
+			Object: endpointspub("private", "svc-change", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		}},
 	}, {
 		Name: "OnCreate-deployment-does-not-exist",
@@ -191,17 +206,17 @@ func TestReconcile(t *testing.T) {
 			SKS("on", "cde", WithDeployRef("blah")),
 			deploy("on", "blah"),
 			// This "has" to pre-exist, otherwise I can't populate it with subsets.
-			endpointspriv("on", "cde", WithSubsets, epsWithName("cde-00001")),
+			endpointspriv("on", "cde", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantCreates: []runtime.Object{
 			svcpriv("on", "cde"),
 			svcpub("on", "cde"),
-			endpointspub("on", "cde", WithSubsets),
+			endpointspub("on", "cde", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("on", "cde", WithDeployRef("blah"),
-				markHappy, WithPubService, WithPrivateService("cde-00001")),
+				markHappy, WithPubService, WithPrivateService),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cde"`),
@@ -211,22 +226,23 @@ func TestReconcile(t *testing.T) {
 		Key:     "update-eps/failA",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("update-eps", "failA", WithPubService, WithPrivateService("failA-abba"), WithDeployRef("blah"), markNoEndpoints),
+			SKS("update-eps", "failA", WithPubService, WithPrivateService, WithDeployRef("blah"), markNoEndpoints),
 			deploy("update-eps", "blah"),
 			svcpub("update-eps", "failA"),
-			svcpriv("update-eps", "failA", svcWithName("failA-abba")),
+			svcpriv("update-eps", "failA"),
 			endpointspub("update-eps", "failA"),
-			endpointspriv("update-eps", "failA", WithSubsets, epsWithName("failA-abba")),
+			endpointspriv("update-eps", "failA", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
 			InduceFailure("update", "endpoints"),
 		},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: endpointspub("update-eps", "failA", WithSubsets), // The attempted update.
+			Object: endpointspub("update-eps", "failA", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)), // The attempted update.
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for update endpoints"),
+			Eventf(corev1.EventTypeWarning, "UpdateFailed",
+				"InternalError: failed to update public K8s Endpoints: inducing failure for update endpoints"),
 		},
 	}, {
 		Name:    "svc-fail-pub",
@@ -235,7 +251,7 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			SKS("svc", "fail2", WithDeployRef("blah")),
 			deploy("svc", "blah"),
-			svcpriv("svc", "fail2", svcWithName("fail2-badbeef")),
+			svcpriv("svc", "fail2"),
 			endpointspriv("svc", "fail2"),
 			activatorEndpoints(WithSubsets),
 		},
@@ -243,13 +259,15 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("create", "services"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("svc", "fail2", WithPrivateService("fail2-badbeef"), WithDeployRef("blah"), markTransitioning("CreatingPublicService")),
+			Object: SKS("svc", "fail2", WithPrivateService,
+				WithDeployRef("blah"), markTransitioning("CreatingPublicService")),
 		}},
 		WantCreates: []runtime.Object{
 			svcpub("svc", "fail2"),
 		},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for create services"),
+			Eventf(corev1.EventTypeWarning, "UpdateFailed",
+				"InternalError: failed to create public K8s Service: inducing failure for create services"),
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "svc/fail2"`),
 		},
 	}, {
@@ -259,54 +277,103 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			SKS("eps", "fail3", WithDeployRef("blah")),
 			deploy("eps", "blah"),
-			svcpriv("eps", "fail3", svcWithName("fail3-abbad")),
-			endpointspriv("eps", "fail3", WithSubsets, epsWithName("fail3-abbad")),
+			svcpriv("eps", "fail3"),
+			endpointspriv("eps", "fail3", WithSubsets),
 			activatorEndpoints(withOtherSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
 			InduceFailure("create", "endpoints"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("eps", "fail3", WithPubService, WithPrivateService("fail3-abbad"), WithDeployRef("blah"),
-				markTransitioning("CreatingPublicEndpoints")),
+			Object: SKS("eps", "fail3", WithPubService, WithPrivateService,
+				WithDeployRef("blah"), markTransitioning("CreatingPublicEndpoints")),
 		}},
 		WantCreates: []runtime.Object{
 			svcpub("eps", "fail3"),
-			endpointspub("eps", "fail3", WithSubsets),
+			endpointspub("eps", "fail3", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for create endpoints"),
+			Eventf(corev1.EventTypeWarning, "UpdateFailed",
+				"InternalError: failed to create public K8s Endpoints: inducing failure for create endpoints"),
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "eps/fail3"`),
 		},
 	}, {
 		Name: "OnCreate-no-eps",
 		Key:  "on/cneps",
 		Objects: []runtime.Object{
-			SKS("on", "cneps", WithDeployRef("blah"), WithPrivateService("cneps-incorrect")),
+			SKS("on", "cneps", WithDeployRef("blah"), WithPrivateService),
 			deploy("on", "blah"),
-			endpointspriv("on", "cneps", epsWithName("cneps-00001")),
+			endpointspriv("on", "cneps"),
 			activatorEndpoints(WithSubsets),
 		},
 		WantCreates: []runtime.Object{
 			svcpriv("on", "cneps"),
 			svcpub("on", "cneps"),
-			endpointspub("on", "cneps", WithSubsets),
+			endpointspub("on", "cneps", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("on", "cneps", WithDeployRef("blah"),
-				markNoEndpoints, WithPubService, WithPrivateService("cneps-00001")),
+				markNoEndpoints, WithPubService, WithPrivateService),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cneps"`),
 		},
 	}, {
-		Name: "OnCreate-no-activator-eps-serve",
+		Name: "OnCreate-no-activator-eps-exist",
+		Key:  "on/cnaeps2",
+		Objects: []runtime.Object{
+			SKS("on", "cnaeps2", WithDeployRef("blah")),
+			deploy("on", "blah"),
+			endpointspriv("on", "cnaeps2", WithSubsets),
+			endpointspub("on", "cnaeps2", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+		},
+		WantErr: true,
+		WantCreates: []runtime.Object{
+			svcpriv("on", "cnaeps2"),
+			svcpub("on", "cnaeps2"),
+		},
+		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: SKS("on", "cnaeps2", WithDeployRef("blah"), WithPubService,
+				WithPrivateService,
+				markTransitioning("CreatingPublicService")),
+		}},
+		WantEvents: []string{
+			Eventf(corev1.EventTypeWarning, "UpdateFailed",
+				`InternalError: failed to get activator service endpoints: endpoints "activator-service" not found`),
+			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cnaeps2"`),
+		},
+	}, {
+		Name: "OnCreate-no-private-eps-exist",
+		Key:  "on/cnaeps3",
+		Objects: []runtime.Object{
+			SKS("on", "cnaeps3", WithDeployRef("blah")),
+			deploy("on", "blah"),
+			endpointspub("on", "cnaeps3", WithSubsets),
+			activatorEndpoints(WithSubsets),
+		},
+		WantErr: true,
+		WantCreates: []runtime.Object{
+			svcpriv("on", "cnaeps3"),
+			svcpub("on", "cnaeps3"),
+		},
+		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: SKS("on", "cnaeps3", WithDeployRef("blah"), WithPubService,
+				WithPrivateService,
+				markTransitioning("CreatingPublicService")),
+		}},
+		WantEvents: []string{
+			Eventf(corev1.EventTypeWarning, "UpdateFailed",
+				`InternalError: failed to get private K8s Service endpoints: endpoints "cnaeps3-private" not found`),
+			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cnaeps3"`),
+		},
+	}, {
+		Name: "OnCreate-no-activator-eps-service",
 		Key:  "on/cnaeps",
 		Objects: []runtime.Object{
 			SKS("on", "cnaeps", WithDeployRef("blah")),
 			deploy("on", "blah"),
-			endpointspriv("on", "cnaeps", WithSubsets, epsWithName("cnaeps-00001")),
-			endpointspub("on", "cnaeps", WithSubsets),
+			endpointspriv("on", "cnaeps", WithSubsets),
+			endpointspub("on", "cnaeps", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 			activatorEndpoints(),
 		},
 		WantCreates: []runtime.Object{
@@ -315,7 +382,7 @@ func TestReconcile(t *testing.T) {
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("on", "cnaeps", WithDeployRef("blah"),
-				markHappy, WithPubService, WithPrivateService("cnaeps-00001")),
+				markHappy, WithPubService, WithPrivateService),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cnaeps"`),
@@ -324,9 +391,9 @@ func TestReconcile(t *testing.T) {
 		Name: "OnCreate-no-activator-eps-proxy",
 		Key:  "on/cnaeps",
 		Objects: []runtime.Object{
-			SKS("on", "cnaeps", WithDeployRef("blah"), WithProxyMode),
+			SKS("on", "cnaeps", WithDeployRef("blah"), withProxyMode),
 			deploy("on", "blah"),
-			endpointspriv("on", "cnaeps", WithSubsets), // This should be ignored.
+			endpointspriv("on", "cnaeps"), // This should be ignored.
 			activatorEndpoints(),
 		},
 		WantCreates: []runtime.Object{
@@ -335,8 +402,8 @@ func TestReconcile(t *testing.T) {
 			endpointspub("on", "cnaeps"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("on", "cnaeps", WithDeployRef("blah"), WithProxyMode,
-				markNoEndpoints, WithPubService, WithPrivateService("cnaeps-00001")),
+			Object: SKS("on", "cnaeps", WithDeployRef("blah"), withProxyMode,
+				markNoEndpoints, WithPubService, WithPrivateService),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cnaeps"`),
@@ -348,7 +415,7 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			SKS("svc", "fail", WithDeployRef("blah")),
 			deploy("svc", "blah"),
-			endpointspriv("svc", "fail", epsWithName("blah-00001")),
+			endpointspriv("svc", "fail"),
 			activatorEndpoints(WithSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
@@ -361,7 +428,8 @@ func TestReconcile(t *testing.T) {
 			svcpriv("svc", "fail"),
 		},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for create services"),
+			Eventf(corev1.EventTypeWarning, "UpdateFailed",
+				"InternalError: failed to create private K8s Service: inducing failure for create services"),
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "svc/fail"`),
 		},
 	}, {
@@ -369,13 +437,13 @@ func TestReconcile(t *testing.T) {
 		Key:     "update-sks/fail4",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("update-sks", "fail4", WithPubService, WithPrivateService("fail4-42x"),
+			SKS("update-sks", "fail4", WithPubService, WithPrivateService,
 				WithDeployRef("blah")),
 			deploy("update-sks", "blah"),
 			svcpub("update-sks", "fail4"),
-			svcpriv("update-sks", "fail4", svcWithName("fail4-42x")),
-			endpointspub("update-sks", "fail4", WithSubsets),
-			endpointspriv("update-sks", "fail4", WithSubsets, epsWithName("fail4-42x")),
+			svcpriv("update-sks", "fail4"),
+			endpointspub("update-sks", "fail4", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+			endpointspriv("update-sks", "fail4", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
@@ -384,7 +452,7 @@ func TestReconcile(t *testing.T) {
 		// We still record update, but it fails.
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("update-sks", "fail4",
-				WithDeployRef("blah"), markHappy, WithPubService, WithPrivateService("fail4-42x")),
+				WithDeployRef("blah"), markHappy, WithPubService, WithPrivateService),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "UpdateFailed", "Failed to update status: inducing failure for update serverlessservices"),
@@ -394,21 +462,21 @@ func TestReconcile(t *testing.T) {
 		Key:     "ronin-priv-service/fail5",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("ronin-priv-service", "fail5", WithPubService, WithPrivateService("fail5-fender"),
+			SKS("ronin-priv-service", "fail5", WithPubService, WithPrivateService,
 				WithDeployRef("blah"), markHappy),
 			deploy("ronin-priv-service", "blah"),
 			svcpub("ronin-priv-service", "fail5"),
-			svcpriv("ronin-priv-service", "fail5", WithK8sSvcOwnersRemoved, svcWithName("fail5-fender")),
+			svcpriv("ronin-priv-service", "fail5", WithK8sSvcOwnersRemoved),
 			endpointspub("ronin-priv-service", "fail5", WithSubsets),
-			endpointspriv("ronin-priv-service", "fail5", WithSubsets, epsWithName("fail5-fender")),
+			endpointspriv("ronin-priv-service", "fail5", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("ronin-priv-service", "fail5", WithPubService, WithPrivateService("fail5-fender"),
-				WithDeployRef("blah"), markUnowned("Service", "fail5-fender")),
+			Object: SKS("ronin-priv-service", "fail5", WithPubService, WithPrivateService,
+				WithDeployRef("blah"), markUnowned("Service", "fail5-private")),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", `InternalError: SKS: fail5 does not own Service: fail5-fender`),
+			Eventf(corev1.EventTypeWarning, "UpdateFailed", `InternalError: SKS: fail5 does not own Service: fail5-private`),
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "ronin-priv-service/fail5"`),
 		},
 	}, {
@@ -416,17 +484,17 @@ func TestReconcile(t *testing.T) {
 		Key:     "ronin-pub-service/fail6",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("ronin-pub-service", "fail6", WithPubService, WithPrivateService("fail6-gibson"),
+			SKS("ronin-pub-service", "fail6", WithPubService, WithPrivateService,
 				WithDeployRef("blah")),
 			deploy("ronin-pub-service", "blah"),
 			svcpub("ronin-pub-service", "fail6", WithK8sSvcOwnersRemoved),
-			svcpriv("ronin-pub-service", "fail6", svcWithName("fail6-gibson")),
+			svcpriv("ronin-pub-service", "fail6"),
 			endpointspub("ronin-pub-service", "fail6", WithSubsets),
-			endpointspriv("ronin-pub-service", "fail6", WithSubsets, epsWithName("fail6-gibson")),
+			endpointspriv("ronin-pub-service", "fail6", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("ronin-pub-service", "fail6", WithPubService, WithPrivateService("fail6-gibson"),
+			Object: SKS("ronin-pub-service", "fail6", WithPubService, WithPrivateService,
 				WithDeployRef("blah"), markUnowned("Service", "fail6")),
 		}},
 		WantEvents: []string{
@@ -438,17 +506,17 @@ func TestReconcile(t *testing.T) {
 		Key:     "ronin-pub-eps/fail7",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("ronin-pub-eps", "fail7", WithPubService, WithPrivateService("fail7-schecter"),
+			SKS("ronin-pub-eps", "fail7", WithPubService, WithPrivateService,
 				WithDeployRef("blah")),
 			deploy("ronin-pub-eps", "blah"),
 			svcpub("ronin-pub-eps", "fail7"),
-			svcpriv("ronin-pub-eps", "fail7", svcWithName("fail7-schecter")),
+			svcpriv("ronin-pub-eps", "fail7"),
 			endpointspub("ronin-pub-eps", "fail7", WithSubsets, WithEndpointsOwnersRemoved),
-			endpointspriv("ronin-pub-eps", "fail7", WithSubsets, epsWithName("fail7-schecter")),
+			endpointspriv("ronin-pub-eps", "fail7", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("ronin-pub-eps", "fail7", WithPubService, WithPrivateService("fail7-schecter"),
+			Object: SKS("ronin-pub-eps", "fail7", WithPubService, WithPrivateService,
 				WithDeployRef("blah"), markUnowned("Endpoints", "fail7")),
 		}},
 		WantEvents: []string{
@@ -460,27 +528,28 @@ func TestReconcile(t *testing.T) {
 		Key:     "update-svc/fail9",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("update-svc", "fail9", WithPubService, WithPrivateService("fail9-yamaha"),
+			SKS("update-svc", "fail9", WithPubService, WithPrivateService,
 				WithDeployRef("blah")),
 			deploy("update-svc", "blah"),
 			svcpub("update-svc", "fail9"),
-			svcpriv("update-svc", "fail9", withTimeSelector, svcWithName("fail9-yamaha")),
+			svcpriv("update-svc", "fail9", withTimeSelector),
 			endpointspub("update-svc", "fail9"),
-			endpointspriv("update-svc", "fail9", epsWithName("fail9-yamaha")),
+			endpointspriv("update-svc", "fail9"),
 			activatorEndpoints(WithSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
 			InduceFailure("update", "services"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("update-svc", "fail9", WithPubService, WithPrivateService("fail9-yamaha"),
+			Object: SKS("update-svc", "fail9", WithPubService, WithPrivateService,
 				WithDeployRef("blah"), markTransitioning("UpdatingPrivateService")),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: svcpriv("update-svc", "fail9", svcWithName("fail9-yamaha")),
+			Object: svcpriv("update-svc", "fail9"),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for update services"),
+			Eventf(corev1.EventTypeWarning, "UpdateFailed",
+				"InternalError: failed to update private K8s Service: inducing failure for update services"),
 			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "update-svc/fail9"`),
 		},
 	},
@@ -489,12 +558,12 @@ func TestReconcile(t *testing.T) {
 			Key:     "update-svc/fail8",
 			WantErr: true,
 			Objects: []runtime.Object{
-				SKS("update-svc", "fail8", WithPubService, WithDeployRef("blah"), markHappy, WithPrivateService("fail8-ibanez")),
+				SKS("update-svc", "fail8", WithPubService, WithDeployRef("blah"), markHappy, WithPrivateService),
 				deploy("update-svc", "blah"),
 				svcpub("update-svc", "fail8", withTimeSelector),
-				svcpriv("update-svc", "fail8", svcWithName("fail8-ibanez")),
+				svcpriv("update-svc", "fail8"),
 				endpointspub("update-svc", "fail8", WithSubsets),
-				endpointspriv("update-svc", "fail8", WithSubsets, epsWithName("fail8-ibanez")),
+				endpointspriv("update-svc", "fail8", WithSubsets),
 				activatorEndpoints(WithSubsets),
 			},
 			WithReactors: []clientgotesting.ReactionFunc{
@@ -504,89 +573,88 @@ func TestReconcile(t *testing.T) {
 				Object: svcpub("update-svc", "fail8"),
 			}},
 			WantEvents: []string{
-				Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for update services"),
+				Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: failed to update public K8s Service: inducing failure for update services"),
 			},
 		}, {
 			Name: "pod change",
 			Key:  "pod/change",
 			Objects: []runtime.Object{
-				SKS("pod", "change", markHappy, WithPubService, WithPrivateService("change-prs"),
+				SKS("pod", "change", markHappy, WithPubService, WithPrivateService,
 					WithDeployRef("blah")),
 				deploy("pod", "blah"),
 				svcpub("pod", "change"),
-				svcpriv("pod", "change", svcWithName("change-prs")),
+				svcpriv("pod", "change"),
 				endpointspub("pod", "change", WithSubsets),
-				endpointspriv("pod", "change", withOtherSubsets, epsWithName("change-prs")),
+				endpointspriv("pod", "change", withOtherSubsets),
 				activatorEndpoints(WithSubsets),
 			},
 			WantUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: endpointspub("pod", "change", withOtherSubsets),
+				Object: endpointspub("pod", "change", withOtherSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 			}},
 		}, {
 			Name: "proxy mode; pod change - activator",
 			Key:  "pod/change",
 			Objects: []runtime.Object{
 				SKS("pod", "change", markNoEndpoints, WithPubService, withHTTP2Protocol,
-					WithPrivateService("change-taylor"), WithProxyMode, WithDeployRef("blah")),
+					WithPrivateService, WithDeployRef("blah")),
 				deploy("pod", "blah"),
 				svcpub("pod", "change", withHTTP2),
-				svcpriv("pod", "change", withHTTP2Priv, svcWithName("change-taylor")),
+				svcpriv("pod", "change", withHTTP2Priv),
 				endpointspub("pod", "change", WithSubsets),
-				endpointspriv("pod", "change", epsWithName("change-taylor")),
+				endpointspriv("pod", "change"),
 				activatorEndpoints(withOtherSubsets),
 			},
 			WantUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: endpointspub("pod", "change", withOtherSubsets),
+				Object: endpointspub("pod", "change", withOtherSubsets, withFilteredPorts(networking.BackendHTTP2Port)),
 			}},
 		}, {
 			Name: "serving mode; serving pod comes online",
 			Key:  "pod/change",
 			Objects: []runtime.Object{
 				SKS("pod", "change", markNoEndpoints, WithPubService,
-					WithPrivateService("change-gretsch"), WithDeployRef("blah")),
+					WithPrivateService, WithDeployRef("blah")),
 				deploy("pod", "blah"),
 				svcpub("pod", "change"),
-				svcpriv("pod", "change", svcWithName("change-gretsch")),
+				svcpriv("pod", "change"),
 				endpointspub("pod", "change", withOtherSubsets),
-				endpointspriv("pod", "change", WithSubsets, epsWithName("change-gretsch")),
+				endpointspriv("pod", "change", WithSubsets),
 				activatorEndpoints(withOtherSubsets),
 			},
 			WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 				Object: SKS("pod", "change",
-					WithDeployRef("blah"), markHappy, WithPubService, WithPrivateService("change-gretsch")),
+					WithDeployRef("blah"), markHappy, WithPubService, WithPrivateService, WithDeployRef("blah")),
 			}},
 			WantEvents: []string{
 				Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "pod/change"`),
 			},
 			WantUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: endpointspub("pod", "change", WithSubsets),
+				Object: endpointspub("pod", "change", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 			}},
 		}, {
 			Name: "serving mode; no backend endpoints",
 			Key:  "pod/change",
 			Objects: []runtime.Object{
 				SKS("pod", "change", WithSKSReady, WithPubService, withHTTP2Protocol,
-					WithPrivateService("change-rickenbecker"), WithDeployRef("blah")),
+					WithPrivateService, WithDeployRef("blah")),
 				deploy("pod", "blah"),
 				svcpub("pod", "change", withHTTP2),
-				svcpriv("pod", "change", withHTTP2Priv, svcWithName("change-rickenbecker")),
-				endpointspub("pod", "change", WithSubsets),                         // We had endpoints...
-				endpointspriv("pod", "change", epsWithName("change-rickenbecker")), // but now we don't.
+				svcpriv("pod", "change", withHTTP2Priv),
+				endpointspub("pod", "change", WithSubsets), // We had endpoints...
+				endpointspriv("pod", "change"),             // but now we don't.
 				activatorEndpoints(withOtherSubsets),
 			},
 			WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 				Object: SKS("pod", "change", withHTTP2Protocol,
-					WithDeployRef("blah"), markNoEndpoints, WithPubService, WithPrivateService("change-rickenbecker")),
+					WithDeployRef("blah"), markNoEndpoints, WithPubService, WithPrivateService),
 			}},
 			WantEvents: []string{
 				Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "pod/change"`),
 			},
 			WantUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: endpointspub("pod", "change", withOtherSubsets),
+				Object: endpointspub("pod", "change", withOtherSubsets, withFilteredPorts(networking.BackendHTTP2Port)),
 			}},
 		}}
 
-	defer logtesting.ClearAll()
 	table.Test(t, MakeFactory(func(ctx context.Context, listers *Listers, cmw configmap.Watcher) controller.Reconciler {
 		return &reconciler{
 			Base:              rpkg.NewBase(ctx, controllerAgentName, cmw),
@@ -598,10 +666,25 @@ func TestReconcile(t *testing.T) {
 	}))
 }
 
+// Keeps only desired port.
+func withFilteredPorts(port int32) EndpointsOption {
+	return func(ep *corev1.Endpoints) {
+		for _, p := range ep.Subsets[0].Ports {
+			if p.Port == port {
+				ep.Subsets[0].Ports[0] = p
+				break
+			}
+		}
+		// Strip all the others.
+		ep.Subsets[0].Ports = ep.Subsets[0].Ports[:1]
+	}
+}
+
 // withOtherSubsets uses different IP set than functional::withSubsets.
 func withOtherSubsets(ep *corev1.Endpoints) {
 	ep.Subsets = []corev1.EndpointSubset{{
 		Addresses: []corev1.EndpointAddress{{IP: "127.0.0.2"}},
+		Ports:     []corev1.EndpointPort{{Port: 8013}, {Port: 8012}},
 	}}
 }
 
@@ -623,6 +706,7 @@ func markTransitioning(s string) SKSOption {
 
 func markNoEndpoints(sks *nv1a1.ServerlessService) {
 	sks.Status.MarkEndpointsNotReady("NoHealthyBackends")
+	sks.Status.MarkActivatorEndpointsPopulated()
 }
 
 func withHTTP2Protocol(sks *nv1a1.ServerlessService) {
@@ -664,6 +748,12 @@ func withHTTP2(svc *corev1.Service) {
 	svc.Spec.Ports[0].TargetPort = intstr.FromInt(networking.BackendHTTP2Port)
 }
 
+// For SKS internal tests this sets mode & activator status.
+func withProxyMode(sks *nv1a1.ServerlessService) {
+	WithProxyMode(sks)
+	sks.Status.MarkActivatorEndpointsPopulated()
+}
+
 func withTargetPortNum(port int) K8sServiceOption {
 	return func(svc *corev1.Service) {
 		svc.Spec.Ports[0].TargetPort = intstr.FromInt(port)
@@ -701,7 +791,7 @@ func activatorEndpoints(eo ...EndpointsOption) *corev1.Endpoints {
 	ep := &corev1.Endpoints{
 		ObjectMeta: metav1.ObjectMeta{
 			Namespace: system.Namespace(),
-			Name:      activator.K8sServiceName,
+			Name:      networking.ActivatorServiceName,
 		},
 	}
 	for _, opt := range eo {
@@ -710,12 +800,6 @@ func activatorEndpoints(eo ...EndpointsOption) *corev1.Endpoints {
 	return ep
 }
 
-func epsWithName(n string) EndpointsOption {
-	return func(e *corev1.Endpoints) {
-		e.Name = n
-	}
-}
-
 func endpointspriv(namespace, name string, eo ...EndpointsOption) *corev1.Endpoints {
 	service := svcpriv(namespace, name)
 	ep := &corev1.Endpoints{
